# Copyright 2026 Arturo Roberti
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# flake8: noqa: F401
from .clone import git_clone
from .types import GitQuery, GitQueryDict
from .url import edit_url, extract_url, is_url, parse_git_query
from .utils import is_git_installed, is_git_repo
from .versioning import (
    commit2version,
    commit_exists,
    determine_ref,
    get_commit,
    get_commit_timestamp,
    get_default_branch,
    get_latest_version,
    version2commit,
)
