# Copyright 2026 Arturo Roberti
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Source helper scripts
for file in $(dirname "${BASH_SOURCE[0]}")/*.bash; do
	if [[ ! "$file" == "${BASH_SOURCE[0]}" ]]; then
		source "$file"
	fi
done

# Ensure this script is not run directly
if [[ "${BASH_SOURCE[0]}" == "${0}" ]]; then
	echo "This is a collection of helpers and cannot be run directly - source it instead" >&2
	exit 1
fi
